# Generated by Django 5.1.7 on 2025-03-31 19:18

from django.db import migrations, models


class Migration(migrations.Migration):
    initial = True

    dependencies = []

    operations = [
        migrations.CreateModel(
            name="ChoicesShellCompleteTester",
            fields=[
                (
                    "id",
                    models.BigAutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                (
                    "char_choice",
                    models.CharField(
                        choices=[
                            ("git", "Git"),
                            ("svn", "Subversion (SVN)"),
                            ("hg", "Mercurial (Hg)"),
                            ("bzr", "Bazaar (Bzr)"),
                            ("cvs", "CVS"),
                            ("perforce", "Perforce"),
                            ("fossil", "Fossil"),
                            ("darcs", "Darcs"),
                            ("monotone", "Monotone"),
                        ],
                        db_index=True,
                        default=None,
                        max_length=12,
                        null=True,
                    ),
                ),
                (
                    "int_choice",
                    models.PositiveSmallIntegerField(
                        choices=[
                            (1, "One"),
                            (2, "Two"),
                            (3, "Mercurial (Hg)"),
                            (4, "Bazaar (Bzr)"),
                            (5, "CVS"),
                            (6, "Perforce"),
                            (7, "Fossil"),
                            (8, "Darcs"),
                            (9, "Monotone"),
                            (10, "Ten"),
                            (11, "Eleven"),
                            (12, "Twelve"),
                        ],
                        db_index=True,
                        default=None,
                        null=True,
                    ),
                ),
                (
                    "ip_choice",
                    models.GenericIPAddressField(
                        choices=[
                            ("192.168.0.1", "Private Residential"),
                            ("10.0.0.1", "Private Gateway"),
                            ("172.16.0.1", "Corporate Network Gateway"),
                            ("8.8.8.8", "Google DNS"),
                            ("127.0.0.1", "Localhost IPv4"),
                            ("::1", "Localhost IPv6"),
                            ("2001:4860:4860::8888", "Google DNS IPv6"),
                            ("fe80::1", "Link-Local IPv6"),
                            ("fd00::1", "Private Network IPv6"),
                            ("2001:db8::1", "Documentation Example IPv6"),
                        ],
                        db_index=True,
                        default=None,
                        null=True,
                    ),
                ),
            ],
        ),
        migrations.CreateModel(
            name="ShellCompleteTester",
            fields=[
                (
                    "id",
                    models.BigAutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                (
                    "char_field",
                    models.CharField(db_index=True, default="", max_length=15),
                ),
                ("text_field", models.TextField(default="")),
                (
                    "float_field",
                    models.FloatField(db_index=True, default=None, null=True),
                ),
                (
                    "decimal_field",
                    models.DecimalField(
                        db_index=True,
                        decimal_places=2,
                        default=None,
                        max_digits=10,
                        null=True,
                    ),
                ),
                ("uuid_field", models.UUIDField(default=None, null=True, unique=True)),
                ("binary_field", models.BinaryField(default=None, null=True)),
                (
                    "ip_field",
                    models.GenericIPAddressField(
                        db_index=True, default=None, null=True
                    ),
                ),
                (
                    "email_field",
                    models.EmailField(
                        db_index=True, default=None, max_length=254, null=True
                    ),
                ),
                ("url_field", models.URLField(db_index=True, default=None, null=True)),
                (
                    "file_field",
                    models.FileField(
                        db_index=True, default=None, null=True, upload_to="documents/"
                    ),
                ),
                (
                    "file_path_field",
                    models.FilePathField(db_index=True, default=None, null=True),
                ),
                (
                    "date_field",
                    models.DateField(db_index=True, default=None, null=True),
                ),
                (
                    "datetime_field",
                    models.DateTimeField(db_index=True, default=None, null=True),
                ),
                (
                    "time_field",
                    models.TimeField(db_index=True, default=None, null=True),
                ),
                (
                    "duration_field",
                    models.DurationField(db_index=True, default=None, null=True),
                ),
            ],
        ),
    ]
