# Generated by Django 5.1.7 on 2025-03-31 19:18

import tests.apps.dj5plus.models
from django.db import migrations, models


class Migration(migrations.Migration):
    initial = True

    dependencies = []

    operations = [
        migrations.CreateModel(
            name="ChoicesShellCompleteTesterDJ5Plus",
            fields=[
                (
                    "id",
                    models.BigAutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                (
                    "char_choice",
                    models.CharField(
                        choices=[
                            ("git", "Git"),
                            ("svn", "Subversion (SVN)"),
                            ("hg", "Mercurial (Hg)"),
                            ("bzr", "Bazaar (Bzr)"),
                            ("cvs", "CVS"),
                            ("perforce", "Perforce"),
                            ("fossil", "Fossil"),
                            ("darcs", "Darcs"),
                            ("monotone", "Monotone"),
                        ],
                        db_index=True,
                        default=None,
                        max_length=12,
                        null=True,
                    ),
                ),
                (
                    "int_choice",
                    models.PositiveSmallIntegerField(
                        choices=[
                            (1, "One"),
                            (2, "Two"),
                            (3, "Mercurial (Hg)"),
                            (4, "Bazaar (Bzr)"),
                            (5, "CVS"),
                            (6, "Perforce"),
                            (7, "Fossil"),
                            (8, "Darcs"),
                            (9, "Monotone"),
                            (10, "Ten"),
                            (11, "Eleven"),
                            (12, "Twelve"),
                        ],
                        db_index=True,
                        default=None,
                        null=True,
                    ),
                ),
                (
                    "ip_choice",
                    models.GenericIPAddressField(
                        choices=tests.apps.dj5plus.models.get_ip_choices,
                        db_index=True,
                        default=None,
                        null=True,
                    ),
                ),
            ],
        ),
    ]
